/* SPDX-FileCopyrightText: 2026 Project Tick
 * SPDX-FileContributor: Project Tick
 * SPDX-License-Identifier: GPL-3.0-or-later WITH LicenseRef-MeshMC-MMCO-Module-Exception-1.0
 *
 *   MeshMC - A Custom Launcher for Minecraft
 *   Copyright (C) 2026 Project Tick
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version, with the additional permission
 *   described in the MeshMC MMCO Module Exception 1.0.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 *   You should have received a copy of the MeshMC MMCO Module Exception 1.0
 *   along with this program.  If not, see <https://projecttick.org/licenses/>.
 */

#pragma once

#include <modplatform/legacy_ftb/PackHelpers.h>
#include <RWStorage.h>

#include <QAbstractListModel>
#include <QSortFilterProxyModel>
#include <QThreadPool>
#include <QIcon>
#include <QStyledItemDelegate>

#include <functional>

namespace LegacyFTB
{

	typedef QMap<QString, QIcon> FTBLogoMap;
	typedef std::function<void(QString)> LogoCallback;

	class FilterModel : public QSortFilterProxyModel
	{
		Q_OBJECT
	  public:
		FilterModel(QObject* parent = Q_NULLPTR);
		enum Sorting { ByName, ByGameVersion };
		const QMap<QString, Sorting> getAvailableSortings();
		QString translateCurrentSorting();
		void setSorting(Sorting sorting);
		Sorting getCurrentSorting();

	  protected:
		bool filterAcceptsRow(int sourceRow,
							  const QModelIndex& sourceParent) const override;
		bool lessThan(const QModelIndex& left,
					  const QModelIndex& right) const override;

	  private:
		QMap<QString, Sorting> sortings;
		Sorting currentSorting;
	};

	class ListModel : public QAbstractListModel
	{
		Q_OBJECT
	  private:
		ModpackList modpacks;
		QStringList m_failedLogos;
		QStringList m_loadingLogos;
		FTBLogoMap m_logoMap;
		QMap<QString, LogoCallback> waitingCallbacks;

		void requestLogo(QString file);
		QString translatePackType(PackType type) const;

	  private slots:
		void logoFailed(QString logo);
		void logoLoaded(QString logo, QIcon out);

	  public:
		ListModel(QObject* parent);
		~ListModel();
		int rowCount(const QModelIndex& parent) const override;
		int columnCount(const QModelIndex& parent) const override;
		QVariant data(const QModelIndex& index, int role) const override;
		Qt::ItemFlags flags(const QModelIndex& index) const override;

		void fill(ModpackList modpacks);
		void addPack(Modpack modpack);
		void clear();
		void remove(int row);

		Modpack at(int row);
		void getLogo(const QString& logo, LogoCallback callback);
	};

} // namespace LegacyFTB
